@@ -39,15 +39,8 @@ yaf_request_t *yaf_request_http_instance(yaf_request_t *this_ptr, zend_string *r
 	if (Z_ISUNDEF_P(this_ptr)) {
 		object_init_ex(this_ptr, yaf_request_http_ce);
 	}
-
-	if (SG(request_info).request_method) {
-		ZVAL_STRING(&method, (char *)SG(request_info).request_method);
-	} else if (strncasecmp(sapi_module.name, "cli", 3)) {
-		ZVAL_STRING(&method, "Unknow");
-	} else {
-		ZVAL_STRING(&method, "Cli");
-	}
-
+	
+	ZVAL_STRING(&method, yaf_request_get_request_method());
 	zend_update_property(yaf_request_http_ce, this_ptr, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_METHOD), &method);
 	zval_ptr_dtor(&method);
 
@@ -122,24 +115,12 @@ yaf_request_t *yaf_request_http_instance(yaf_request_t *this_ptr, zend_string *r
 				if (EXPECTED(Z_TYPE_P(uri) == IS_STRING)) {
 					settled_uri = zend_string_copy(Z_STR_P(uri));
 					break;
-				} 
+				}
 			}
 		} while (0);
 	}
 
 	if (settled_uri) {
-		char *p = ZSTR_VAL(settled_uri);
-
-		while (*p == '/' && *(p + 1) == '/') {
-			p++;
-		}
-
-		if (p != ZSTR_VAL(settled_uri)) {
-			zend_string *garbage = settled_uri;
-			settled_uri = zend_string_init(p, ZSTR_LEN(settled_uri) - (p - ZSTR_VAL(settled_uri)), 0);
-			zend_string_release(garbage);
-		}
-
 		zend_update_property_str(yaf_request_http_ce, this_ptr, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_URI), settled_uri);
 		yaf_request_set_base_uri(this_ptr, base_uri, settled_uri);
 		zend_string_release(settled_uri);
@@ -282,8 +263,8 @@ PHP_METHOD(yaf_request_http, __construct) {
 	if (zend_parse_parameters_throw(ZEND_NUM_ARGS(), "|SS", &request_uri, &base_uri) == FAILURE) {
 		return;
 	}
-	
-	(void)yaf_request_http_instance(self, request_uri, base_uri);
+
+	yaf_request_http_instance(self, request_uri, base_uri);
 }
 /* }}} */
 
