@@ -14,13 +14,12 @@
   +----------------------------------------------------------------------+
 */
 
-
 #ifndef YAF_ROUTER_REWRITE_H
 #define YAF_ROUTER_REWRITE_H
 
 extern zend_class_entry *yaf_route_rewrite_ce;
 
-yaf_route_t * yaf_route_rewrite_instance(yaf_route_t *this_ptr, zval *match, zval *router, zval *verify TSRMLS_DC);
+yaf_route_t * yaf_route_rewrite_instance(yaf_route_t *this_ptr, zval *match, zval *router, zval *verify);
 
 YAF_STARTUP_FUNCTION(route_rewrite);
 
