@@ -71,9 +71,6 @@ PHP_INI_BEGIN()
 	STD_PHP_INI_ENTRY("yaf.forward_limit", 		"5", PHP_INI_ALL, OnUpdateLongGEZero, forward_limit, zend_yaf_globals, yaf_globals)
 	STD_PHP_INI_BOOLEAN("yaf.name_suffix", 		"1", PHP_INI_ALL, OnUpdateBool, name_suffix, zend_yaf_globals, yaf_globals)
 	PHP_INI_ENTRY("yaf.name_separator", 		"",  PHP_INI_ALL, OnUpdateSeparator)
-/* {{{ This only effects internally */
-	STD_PHP_INI_BOOLEAN("yaf.st_compatible",     "0", PHP_INI_ALL, OnUpdateBool, st_compatible, zend_yaf_globals, yaf_globals)
-/* }}} */
 	STD_PHP_INI_ENTRY("yaf.environ",        	"product", PHP_INI_SYSTEM, OnUpdateString, environ_name, zend_yaf_globals, yaf_globals)
 	STD_PHP_INI_BOOLEAN("yaf.use_namespace",   	"0", PHP_INI_SYSTEM, OnUpdateBool, use_namespace, zend_yaf_globals, yaf_globals)
 PHP_INI_END();
@@ -195,16 +192,30 @@ PHP_RSHUTDOWN_FUNCTION(yaf)
 		YAF_G(local_library) = NULL;
 	}
 	if (YAF_G(local_namespaces)) {
-		zend_string_release(YAF_G(local_namespaces));
+#if PHP_VERSION_ID >70300
+		if (zend_gc_delref(YAF_G(local_namespaces)) == 0)
+#else
+		if (--(GC_REFCOUNT(YAF_G(local_namespaces))) == 0)
+#endif
+		{
+			zend_array_destroy(YAF_G(local_namespaces));
+		}
 		YAF_G(local_namespaces) = NULL;
 	}
 	if (YAF_G(bootstrap)) {
 		zend_string_release(YAF_G(bootstrap));
 		YAF_G(bootstrap) = NULL;
 	}
-	if (Z_TYPE(YAF_G(modules)) == IS_ARRAY) {
-		zval_ptr_dtor(&YAF_G(modules));
-		ZVAL_UNDEF(&YAF_G(modules));
+	if (YAF_G(modules)) {
+#if PHP_VERSION_ID >70300
+		if (zend_gc_delref(YAF_G(modules)) == 0)
+#else
+		if (--(GC_REFCOUNT(YAF_G(modules))) == 0)
+#endif
+		{
+			zend_array_destroy(YAF_G(modules));
+		}
+		YAF_G(modules) = NULL;
 	}
 	if (YAF_G(base_uri)) {
 		zend_string_release(YAF_G(base_uri));
