@@ -77,21 +77,33 @@ ZEND_BEGIN_ARG_INFO_EX(yaf_application_setappdir_arginfo, 0, 0, 1)
 ZEND_END_ARG_INFO()
 /* }}} */
 
-int yaf_application_is_module_name(zend_string *name) /* {{{ */ {
-	zval *modules, *pzval;
+static inline HashTable* yaf_application_get_module_nams() /* {{{ */ {
+	zval *modules;
 	yaf_application_t *app;
 
 	app = zend_read_static_property(yaf_application_ce, ZEND_STRL(YAF_APPLICATION_PROPERTY_NAME_APP), 1);
 	if (UNEXPECTED(Z_TYPE_P(app) != IS_OBJECT)) {
-		return 0;
+		return NULL;
 	}
 
 	modules = zend_read_property(yaf_application_ce, app, ZEND_STRL(YAF_APPLICATION_PROPERTY_NAME_MODULES), 1, NULL);
 	if (UNEXPECTED(Z_TYPE_P(modules) != IS_ARRAY)) {
+		return NULL;
+	}
+
+	return Z_ARRVAL_P(modules);
+}
+/* }}} */
+
+int yaf_application_is_module_name(zend_string *name) /* {{{ */ {
+	zval *pzval;
+	HashTable *modules = yaf_application_get_module_nams();
+
+	if (modules == NULL) {
 		return 0;
 	}
 
-	ZEND_HASH_FOREACH_VAL(Z_ARRVAL_P(modules), pzval) {
+	ZEND_HASH_FOREACH_VAL(modules, pzval) {
 		if (UNEXPECTED(Z_TYPE_P(pzval) != IS_STRING)) {
 			continue;
 		}
@@ -103,11 +115,24 @@ int yaf_application_is_module_name(zend_string *name) /* {{{ */ {
 }
 /* }}} */
 
-int yaf_application_is_module_name_str(char *name, size_t len) /* {{{ */ {
-	zend_string *str = zend_string_init(name, len, 0);
-	int ret = yaf_application_is_module_name(str);
-	zend_string_release(str);
-	return ret;
+int yaf_application_is_module_name_str(const char *name, size_t len) /* {{{ */ {
+	zval *pzval;
+	HashTable *modules = yaf_application_get_module_nams();
+
+	if (modules == NULL) {
+		return 0;
+	}
+
+	ZEND_HASH_FOREACH_VAL(modules, pzval) {
+		if (UNEXPECTED(Z_TYPE_P(pzval) != IS_STRING)) {
+			continue;
+		}
+		if (Z_STRLEN_P(pzval) == len && strncasecmp(Z_STRVAL_P(pzval), name, len) == 0) {
+			return 1;
+		}
+	} ZEND_HASH_FOREACH_END();
+
+	return 0;
 }
 /* }}} */
 
@@ -153,37 +178,33 @@ static int yaf_application_parse_option(zval *options) /* {{{ */ {
 
 	if (EXPECTED((pzval = zend_hash_str_find(Z_ARRVAL_P(app), ZEND_STRL("library"))) != NULL)) {
 		if (IS_STRING == Z_TYPE_P(pzval)) {
-			if (*(Z_STRVAL_P(pzval) + Z_STRLEN_P(pzval)) == DEFAULT_SLASH) {
-				YAF_G(local_library) = zend_string_init(Z_STRVAL_P(pzval), Z_STRLEN_P(pzval) - 1, 0);
-			} else {
-				YAF_G(local_library) = zend_string_copy(Z_STR_P(pzval));
-			}
+			YAF_G(local_library) = zend_string_copy(Z_STR_P(pzval));
 		} else if (IS_ARRAY == Z_TYPE_P(pzval)) {
 			if ((psval = zend_hash_str_find(Z_ARRVAL_P(pzval),
-							ZEND_STRL("directory"))) != NULL && Z_TYPE_P(psval) == IS_STRING) {
-				if (*(Z_STRVAL_P(psval) + Z_STRLEN_P(psval)) == DEFAULT_SLASH) {
-					YAF_G(local_library) = zend_string_init(Z_STRVAL_P(psval), Z_STRLEN_P(psval) - 1, 0);
-				} else {
-					YAF_G(local_library) = zend_string_copy(Z_STR_P(psval));
-				}
+				ZEND_STRL("directory"))) != NULL && Z_TYPE_P(psval) == IS_STRING) {
+				YAF_G(local_library) = zend_string_copy(Z_STR_P(psval));
 			}
 			if ((psval = zend_hash_str_find(Z_ARRVAL_P(pzval),
-							ZEND_STRL("namespace"))) != NULL && Z_TYPE_P(psval) == IS_STRING) {
-				uint i, len;
-				char *src = Z_STRVAL_P(psval);
+						ZEND_STRL("namespace"))) != NULL && Z_TYPE_P(psval) == IS_STRING) {
 				if (Z_STRLEN_P(psval)) {
-				    char *target = emalloc(Z_STRLEN_P(psval) + 1);
-					len = 0;
-					for(i=0; i<Z_STRLEN_P(psval); i++) {
-						if (src[i] == ',') {
-							target[len++] = DEFAULT_DIR_SEPARATOR;
-						} else if (src[i] != ' ') {
-							target[len++] = src[i];
-						}
+					zend_string *prefix;
+					char *src = Z_STRVAL_P(psval), *pos;
+					size_t len = Z_STRLEN_P(psval);
+					while ((pos = memchr(src, ',', len))) {
+						len -= pos - src;
+						while (*src == ' ') src++;
+						while (*pos == ' ') pos--;
+						prefix = zend_string_init(src, pos - src, 0);
+						yaf_loader_register_namespace_single(prefix);
+						zend_string_release(prefix);
+						src = pos + 1;
+					}
+
+					if (len) {
+						prefix = zend_string_init(src, len, 0);
+						yaf_loader_register_namespace_single(prefix);
+						zend_string_release(prefix);
 					}
-					target[len] = '\0';
-					yaf_loader_register_namespace_single(target, len);
-					efree(target);
 				}
 			}
 		}
@@ -238,7 +259,7 @@ static int yaf_application_parse_option(zval *options) /* {{{ */ {
 		if ((psval = zend_hash_str_find(Z_ARRVAL_P(pzval),
 						ZEND_STRL("defaultRoute"))) != NULL && Z_TYPE_P(psval) == IS_ARRAY) {
 			/* increase the refcount? */
-			YAF_G(default_route) = psval;
+			YAF_G(default_route) = Z_ARRVAL_P(psval);
 		}
 	}
 
@@ -246,9 +267,11 @@ static int yaf_application_parse_option(zval *options) /* {{{ */ {
 		char *ptrptr;
 		zval module;
 
-		array_init(&YAF_G(modules));
+		ALLOC_HASHTABLE(YAF_G(modules));
+		zend_hash_init(YAF_G(modules), 8, NULL, ZVAL_PTR_DTOR, 0);
+
 		if ((pzval = zend_hash_str_find(Z_ARRVAL_P(app),
-						ZEND_STRL("modules"))) != NULL && Z_TYPE_P(pzval) == IS_STRING && Z_STRLEN_P(pzval)) {
+				ZEND_STRL("modules"))) != NULL && Z_TYPE_P(pzval) == IS_STRING && Z_STRLEN_P(pzval)) {
 			char *seg, *modules;
 			modules = estrndup(Z_STRVAL_P(pzval), Z_STRLEN_P(pzval));
 			seg = php_strtok_r(modules, ",", &ptrptr);
@@ -256,19 +279,19 @@ static int yaf_application_parse_option(zval *options) /* {{{ */ {
 				if (seg && strlen(seg)) {
 					ZVAL_STRINGL(&module, seg, strlen(seg));
 					*(Z_STRVAL(module)) = toupper(*Z_STRVAL(module));
-					zend_hash_next_index_insert(Z_ARRVAL(YAF_G(modules)), &module);
+					zend_hash_next_index_insert(YAF_G(modules), &module);
 				}
 				seg = php_strtok_r(NULL, ",", &ptrptr);
 			}
 			efree(modules);
 		} else {
 			ZVAL_STR_COPY(&module, YAF_G(default_module));
-			zend_hash_next_index_insert(Z_ARRVAL(YAF_G(modules)), &module);
+			zend_hash_next_index_insert(YAF_G(modules), &module);
 		}
 	} while (0);
 
 	if ((pzval = zend_hash_str_find(Z_ARRVAL_P(app),
-					ZEND_STRL("system"))) != NULL && Z_TYPE_P(pzval) == IS_ARRAY) {
+			ZEND_STRL("system"))) != NULL && Z_TYPE_P(pzval) == IS_ARRAY) {
 		zval *value;
 		char name[128];
 		zend_string *key;
@@ -297,12 +320,12 @@ static int yaf_application_parse_option(zval *options) /* {{{ */ {
 */
 PHP_METHOD(yaf_application, __construct) {
 	zval *config;
-	zval *section = NULL, zsection = {{0}};
+	zval *section = NULL;
 	yaf_config_t zconfig = {{0}};
 	yaf_request_t zrequest = {{0}};
 	yaf_dispatcher_t zdispatcher = {{0}};
-	yaf_application_t *app, *self;
 	yaf_loader_t *loader, zloader = {{0}};
+	yaf_application_t *app, *self;
 
 #if PHP_YAF_DEBUG
 	php_error_docref(NULL, E_STRICT, "Yaf is running in debug mode");
@@ -321,16 +344,16 @@ PHP_METHOD(yaf_application, __construct) {
 
 	self = getThis();
 	if (!section || Z_TYPE_P(section) != IS_STRING || !Z_STRLEN_P(section)) {
+		zval zsection;
 		ZVAL_STRING(&zsection, YAF_G(environ_name));
-		(void)yaf_config_instance(&zconfig, config, &zsection);
+		yaf_config_instance(&zconfig, config, &zsection);
 		zval_ptr_dtor(&zsection);
 	} else {
-		(void)yaf_config_instance(&zconfig, config, section);
+		yaf_config_instance(&zconfig, config, section);
 	}
 
-	if  (UNEXPECTED(Z_TYPE(zconfig) != IS_OBJECT
-			|| yaf_application_parse_option(zend_read_property(yaf_config_ce,
-					&zconfig, ZEND_STRL(YAF_CONFIG_PROPERT_NAME), 1, NULL)) == FAILURE)) {
+	if (UNEXPECTED(Z_TYPE(zconfig) != IS_OBJECT ||
+			yaf_application_parse_option(zend_read_property(yaf_config_ce, &zconfig, ZEND_STRL(YAF_CONFIG_PROPERT_NAME), 1, NULL)) == FAILURE)) {
 		yaf_trigger_error(YAF_ERR_STARTUP_FAILED, "Initialization of application config failed");
 		zval_ptr_dtor(&zconfig);
 		RETURN_FALSE;
@@ -382,11 +405,16 @@ PHP_METHOD(yaf_application, __construct) {
 	}
 
 	zend_update_property_bool(yaf_application_ce, self, ZEND_STRL(YAF_APPLICATION_PROPERTY_NAME_RUN), 0);
-	zend_update_property_string(yaf_application_ce, self, ZEND_STRL(YAF_APPLICATION_PROPERTY_NAME_ENV), YAF_G(environ_name));
+	if (section && Z_TYPE_P(section) == IS_STRING && Z_STRLEN_P(section)) {
+		zend_update_property_str(yaf_application_ce, self, ZEND_STRL(YAF_APPLICATION_PROPERTY_NAME_ENV), Z_STR_P(section));
+	} else {
+		zend_update_property_string(yaf_application_ce, self, ZEND_STRL(YAF_APPLICATION_PROPERTY_NAME_ENV), YAF_G(environ_name));
+	}
 
-	if (Z_TYPE(YAF_G(modules)) == IS_ARRAY) {
-		zend_update_property(yaf_application_ce,
-				self, ZEND_STRL(YAF_APPLICATION_PROPERTY_NAME_MODULES), &YAF_G(modules));
+	if (YAF_G(modules)) {
+		zval rv;
+		ZVAL_ARR(&rv, YAF_G(modules));
+		zend_update_property(yaf_application_ce, self, ZEND_STRL(YAF_APPLICATION_PROPERTY_NAME_MODULES), &rv);
 	} else {
 		zend_update_property_null(yaf_application_ce, self, ZEND_STRL(YAF_APPLICATION_PROPERTY_NAME_MODULES));
 	}
@@ -516,20 +544,20 @@ PHP_METHOD(yaf_application, environ) {
 /** {{{ proto public Yaf_Application::bootstrap(void)
 */
 PHP_METHOD(yaf_application, bootstrap) {
-	zend_string	*bootstrap_path;
-	uint  retval = 1;
+	unsigned  retval = 1;
 	zend_class_entry  *ce;
 	yaf_application_t *self = getThis();
 
 	if (!(ce = zend_hash_str_find_ptr(EG(class_table),
 					YAF_DEFAULT_BOOTSTRAP_LOWER, sizeof(YAF_DEFAULT_BOOTSTRAP_LOWER) - 1))) {
+		zend_string *bootstrap_path;
 		if (YAF_G(bootstrap)) {
 			bootstrap_path = zend_string_copy(YAF_G(bootstrap));
 		} else {
 			bootstrap_path = strpprintf(0, "%s%c%s.%s",
 					ZSTR_VAL(YAF_G(directory)), DEFAULT_SLASH, YAF_DEFAULT_BOOTSTRAP, ZSTR_VAL(YAF_G(ext)));
 		}
-		if (!yaf_loader_import(bootstrap_path, 0)) {
+		if (!yaf_loader_import(ZSTR_VAL(bootstrap_path), ZSTR_LEN(bootstrap_path))) {
 			php_error_docref(NULL, E_WARNING, "Couldn't find bootstrap file %s", ZSTR_VAL(bootstrap_path));
 			retval = 0;
 		} else if (UNEXPECTED((ce = zend_hash_str_find_ptr(EG(class_table),
