@@ -29,7 +29,7 @@
 extern zend_class_entry *yaf_application_ce;
 
 int yaf_application_is_module_name(zend_string *name);
-int yaf_application_is_module_name_str(char *name, size_t len);
+int yaf_application_is_module_name_str(const char *name, size_t len);
 
 YAF_STARTUP_FUNCTION(application);
 #endif
