@@ -143,10 +143,10 @@ yaf_dispatcher_t *yaf_dispatcher_instance(yaf_dispatcher_t *this_ptr) /* {{{ */
 }
 /* }}} */
 
-static void yaf_dispatcher_get_call_parameters(zend_class_entry *request_ce, yaf_request_t *request, zend_function *fptr, zval **params, uint *count) /* {{{ */ {
+static void yaf_dispatcher_get_call_parameters(zend_class_entry *request_ce, yaf_request_t *request, zend_function *fptr, zval **params, unsigned *count) /* {{{ */ {
 	zval          *args, *arg;
 	zend_arg_info *arg_info;
-	uint           current;
+	unsigned       current;
 	HashTable 	  *params_ht;
 
 	args = zend_read_property(request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_PARAMS), 1, NULL);
@@ -159,13 +159,12 @@ static void yaf_dispatcher_get_call_parameters(zend_class_entry *request_ce, yaf
 			ZVAL_COPY_VALUE(&((*params)[current]), arg);
 			(*count)++;
 		} else {
+#if 0 /* is this really useful out of there? */
 			zend_string *key;
-
-			arg  = NULL;
 			/* since we need search ignoring case, can't use zend_hash_find */
 			ZEND_HASH_FOREACH_STR_KEY_VAL(params_ht, key, arg) {
                 if (key) {
-					if (zend_string_equals(key, arg_info->name)) {
+					if (zend_string_equals_ci(key, arg_info->name)) {
 							/* return when we find first match, there is a trap
 							 * when multi different parameters in different case presenting in params_ht
 							 * only the first take affect
@@ -175,11 +174,15 @@ static void yaf_dispatcher_get_call_parameters(zend_class_entry *request_ce, yaf
 							break;
 					}
 				}
+				arg = NULL;
 			} ZEND_HASH_FOREACH_END();
 
-			if (NULL == arg) {
+			if (arg == NULL) {
 				break;
 			}
+#else
+			break;
+#endif
 		}
 	}
 }
@@ -280,215 +283,150 @@ int yaf_dispatcher_set_request(yaf_dispatcher_t *dispatcher, yaf_request_t *requ
 /* }}} */
 
 zend_class_entry *yaf_dispatcher_get_controller(zend_string *app_dir, zend_string *module, zend_string *controller, int def_module) /* {{{ */ {
-	char *directory;
+	char directory[MAXPATHLEN];
 	size_t directory_len;
+	zend_class_entry *ce;
+	zend_string *lc_name;
 
 	if (def_module) {
-		directory_len = spprintf(&directory, 0,
+		directory_len = snprintf(directory, sizeof(directory),
 				"%s%c%s", ZSTR_VAL(app_dir), DEFAULT_SLASH, YAF_CONTROLLER_DIRECTORY_NAME);
 	} else {
-		directory_len = spprintf(&directory, 0,
+		directory_len = snprintf(directory, sizeof(directory),
 				"%s%c%s%c%s%c%s", ZSTR_VAL(app_dir), DEFAULT_SLASH, YAF_MODULE_DIRECTORY_NAME,
 				DEFAULT_SLASH, ZSTR_VAL(module), DEFAULT_SLASH, YAF_CONTROLLER_DIRECTORY_NAME);
 	}
 
-	if (EXPECTED(directory_len)) {
-		zend_string *class;
-		zend_string *class_lowercase;
-		zend_class_entry *ce 	= NULL;
-
-		if (YAF_G(name_suffix)) {
-			class = strpprintf(0, "%s%s%s", ZSTR_VAL(controller), YAF_G(name_separator), "Controller");
-		} else {
-			class = strpprintf(0, "%s%s%s", "Controller", YAF_G(name_separator), ZSTR_VAL(controller));
-		}
-
-		class_lowercase = zend_string_tolower(class);
-
-		if ((ce = zend_hash_find_ptr(EG(class_table), class_lowercase)) == NULL) {
-			if (!yaf_internal_autoload(ZSTR_VAL(controller), ZSTR_LEN(controller), &directory)) {
-				yaf_trigger_error(YAF_ERR_NOTFOUND_CONTROLLER,
-						"Failed opening controller script %s: %s", directory, strerror(errno));
-				zend_string_release(class);
-				zend_string_release(class_lowercase);
-				efree(directory);
-				return NULL;
-			} else if ((ce = zend_hash_find_ptr(EG(class_table), class_lowercase)) == NULL)  {
-				yaf_trigger_error(YAF_ERR_AUTOLOAD_FAILED,
-						"Could not find class %s in controller script %s", ZSTR_VAL(class), directory);
-				zend_string_release(class);
-				zend_string_release(class_lowercase);
-				efree(directory);
-				return 0;
-			} else if (!instanceof_function(ce, yaf_controller_ce)) {
-				yaf_trigger_error(YAF_ERR_TYPE_ERROR,
-						"Controller must be an instance of %s", ZSTR_VAL(yaf_controller_ce->name));
-				zend_string_release(class);
-				zend_string_release(class_lowercase);
-				efree(directory);
-				return 0;
-			}
-		}
-
-		zend_string_release(class);
-		zend_string_release(class_lowercase);
-		efree(directory);
-
-		return ce;
+	if (directory_len >= sizeof(directory)) {
+		yaf_trigger_error(YAF_ERR_AUTOLOAD_FAILED, "path too long %s: %s", directory);
+		return NULL;
 	}
 
-	return NULL;
+	lc_name = zend_string_alloc(ZSTR_LEN(controller) + YAF_G(name_separator_len) + sizeof("Controller") - 1, 0);
+	if (EXPECTED(YAF_G(name_suffix))) {
+		char *p = ZSTR_VAL(lc_name);
+		zend_str_tolower_copy(p, ZSTR_VAL(controller), ZSTR_LEN(controller));
+		p += ZSTR_LEN(controller);
+		if (UNEXPECTED(YAF_G(name_separator_len))) {
+			zend_str_tolower_copy(p, YAF_G(name_separator), YAF_G(name_separator_len));
+			p += YAF_G(name_separator_len);
+		}
+		memcpy(p, "controller", sizeof("controller"));
+	} else {
+		char *p = ZSTR_VAL(lc_name);
+		memcpy(p, "controller", sizeof("controller") - 1);
+		p += sizeof("controller") - 1;
+		if (UNEXPECTED(YAF_G(name_separator_len))) {
+			zend_str_tolower_copy(p, YAF_G(name_separator), YAF_G(name_separator_len));
+			p += YAF_G(name_separator_len);
+		}
+		zend_str_tolower_copy(p, ZSTR_VAL(controller), ZSTR_LEN(controller) + 1);
+	}
+
+	if ((ce = (zend_class_entry*)zend_hash_find_ptr(EG(class_table), lc_name)) == NULL) {
+		if (!yaf_loader_load(NULL, ZSTR_VAL(controller), ZSTR_LEN(controller), directory, directory_len)) {
+			yaf_trigger_error(YAF_ERR_NOTFOUND_CONTROLLER,
+					"Failed opening controller script %s: %s", directory, strerror(errno));
+			zend_string_release(lc_name);
+			return NULL;
+		} else if ((ce = zend_hash_find_ptr(EG(class_table), lc_name)) == NULL)  {
+			zend_string_release(lc_name);
+			if (EXPECTED(YAF_G(name_suffix))) {
+				yaf_trigger_error(YAF_ERR_AUTOLOAD_FAILED,
+					"Could not find class %s%s%s in controller script %s",
+					ZSTR_VAL(controller), YAF_G(name_separator), "Controller", directory);
+			} else {
+				yaf_trigger_error(YAF_ERR_AUTOLOAD_FAILED,
+					"Could not find class %s%s%s in controller script %s",
+					"Controller", YAF_G(name_separator), ZSTR_VAL(controller), directory);
+			}
+			return 0;
+		} else if (!instanceof_function(ce, yaf_controller_ce)) {
+			yaf_trigger_error(YAF_ERR_TYPE_ERROR,
+					"Controller must be an instance of %s", ZSTR_VAL(yaf_controller_ce->name));
+			zend_string_release(lc_name);
+			return 0;
+		}
+	}
+
+	zend_string_release(lc_name);
+
+	return ce;
+
 }
 /* }}} */
 
 zend_class_entry *yaf_dispatcher_get_action(zend_string *app_dir, yaf_controller_t *controller, char *module, int def_module, zend_string *action) /* {{{ */ {
 	zval *paction, *actions_map;
-	actions_map = zend_read_property(Z_OBJCE_P(controller),
-			controller, ZEND_STRL(YAF_CONTROLLER_PROPERTY_NAME_ACTIONS), 1, NULL);
+	actions_map = zend_read_property(Z_OBJCE_P(controller), controller, ZEND_STRL(YAF_CONTROLLER_PROPERTY_NAME_ACTIONS), 1, NULL);
+
+	ZVAL_DEREF(actions_map);
 
 	if (EXPECTED(IS_ARRAY == Z_TYPE_P(actions_map))) {
 		zend_class_entry *ce;
-		zend_string *class;
-		zend_string *class_lowercase;
-		char *action_upper = estrndup(ZSTR_VAL(action), ZSTR_LEN(action));
+		zend_string *lc_name;
 
-		*(action_upper) = toupper(*action_upper);
-
-		if (YAF_G(name_suffix)) {
-			class = strpprintf(0, "%s%s%s", action_upper, YAF_G(name_separator), "Action");
+		lc_name = zend_string_alloc(ZSTR_LEN(action) + YAF_G(name_separator_len) + sizeof("Action") - 1, 0);
+		if (EXPECTED(YAF_G(name_suffix))) {
+			char *p = ZSTR_VAL(lc_name);
+			zend_str_tolower_copy(p, ZSTR_VAL(action), ZSTR_LEN(action));
+			p += ZSTR_LEN(action);
+			if (UNEXPECTED(YAF_G(name_separator_len))) {
+				zend_str_tolower_copy(p, YAF_G(name_separator), YAF_G(name_separator_len));
+				p += YAF_G(name_separator_len);
+			}
+			memcpy(p, "action", sizeof("action"));
 		} else {
-			class = strpprintf(0, "%s%s%s", "Action", YAF_G(name_separator), action_upper);
+			char *p = ZSTR_VAL(lc_name);
+			memcpy(p, "action", sizeof("action"));
+			p += sizeof("action") - 1;
+			if (UNEXPECTED(YAF_G(name_separator_len))) {
+				zend_str_tolower_copy(p, YAF_G(name_separator), YAF_G(name_separator_len));
+				p += YAF_G(name_separator_len);
+			}
+			zend_str_tolower_copy(p, ZSTR_VAL(action), ZSTR_LEN(action) + 1);
 		}
 
-		class_lowercase = zend_string_tolower(class);
-
-		if ((ce = zend_hash_find_ptr(EG(class_table), class_lowercase)) != NULL) {
-			efree(action_upper);
-			zend_string_release(class_lowercase);
+		if ((ce = zend_hash_find_ptr(EG(class_table), lc_name)) != NULL) {
+			zend_string_release(lc_name);
 			if (instanceof_function(ce, yaf_action_ce)) {
-				zend_string_release(class);
 				return ce;
 			} else {
 				yaf_trigger_error(YAF_ERR_TYPE_ERROR,
-						"Action %s must extends from %s", ZSTR_VAL(class), ZSTR_VAL(yaf_action_ce->name));
-				zend_string_release(class);
+						"Action %s must extends from %s", ZSTR_VAL(action), ZSTR_VAL(yaf_action_ce->name));
 				return NULL;
 			}
 		}
 
 		if ((paction = zend_hash_find(Z_ARRVAL_P(actions_map), action)) != NULL) {
 			zend_string *action_path;
+			ZVAL_DEREF(paction);
 
 			action_path = strpprintf(0, "%s%c%s", ZSTR_VAL(app_dir), DEFAULT_SLASH, Z_STRVAL_P(paction));
-			if (yaf_loader_import(action_path, 0)) {
-				if ((ce = zend_hash_find_ptr(EG(class_table), class_lowercase)) != NULL) {
-					zend_string_release(action_path);
-					efree(action_upper);
-					zend_string_release(class_lowercase);
-
+			if (yaf_loader_import(ZSTR_VAL(action_path), ZSTR_LEN(action_path))) {
+				if ((ce = zend_hash_find_ptr(EG(class_table), lc_name)) != NULL) {
 					if (instanceof_function(ce, yaf_action_ce)) {
-						zend_string_release(class);
+						zend_string_release(action_path);
+						zend_string_release(lc_name);
 						return ce;
 					} else {
 						yaf_trigger_error(YAF_ERR_TYPE_ERROR,
-								"Action %s must extends from %s", ZSTR_VAL(class), ZSTR_VAL(yaf_action_ce->name));
-						zend_string_release(class);
+								"Action %s must extends from %s", ZSTR_VAL(action), ZSTR_VAL(yaf_action_ce->name));
 					}
-
 				} else {
 					yaf_trigger_error(YAF_ERR_NOTFOUND_ACTION,
-							"Could not find action %s in %s", ZSTR_VAL(class), ZSTR_VAL(action_path));
+							"Could not find action %s in %s", ZSTR_VAL(action), ZSTR_VAL(action_path));
 				}
-
-				zend_string_release(action_path);
-				efree(action_upper);
-				zend_string_release(class);
-				zend_string_release(class_lowercase);
 			} else {
 				yaf_trigger_error(YAF_ERR_NOTFOUND_ACTION,
 						"Failed opening action script %s: %s", ZSTR_VAL(action_path), strerror(errno));
-				zend_string_release(action_path);
 			}
+			zend_string_release(action_path);
 		} else {
 			yaf_trigger_error(YAF_ERR_NOTFOUND_ACTION, "There is no method %s%s in %s::$%s", ZSTR_VAL(action),
 					"Action", ZSTR_VAL(Z_OBJCE_P(controller)->name), YAF_CONTROLLER_PROPERTY_NAME_ACTIONS);
 		}
-	} else
-/* {{{ This only effects internally */
-	   	if (YAF_G(st_compatible)) {
-		char *directory, *class, *class_lowercase, *p;
-		uint class_len;
-		zend_class_entry *ce;
-		char *action_upper = estrndup(ZSTR_VAL(action), ZSTR_LEN(action));
-
-		/**
-		 * upper Action Name
-		 * eg: Index_sub -> Index_Sub
-		 */
-		p = action_upper;
-		*(p) = toupper(*p);
-		while (*p != '\0') {
-			if (*p == '_' || *p == '\\') {
-				if (*(p+1) != '\0') {
-					*(p+1) = toupper(*(p+1));
-					p++;
-				}
-			}
-			p++;
-		}
-
-		if (def_module) {
-			spprintf(&directory, 0, "%s%c%s", ZSTR_VAL(app_dir), DEFAULT_SLASH, "actions");
-		} else {
-			spprintf(&directory, 0, "%s%c%s%c%s%c%s", ZSTR_VAL(app_dir), DEFAULT_SLASH,
-					"modules", DEFAULT_SLASH, module, DEFAULT_SLASH, "actions");
-		}
-
-		if (YAF_G(name_suffix)) {
-			class_len = spprintf(&class, 0, "%s%s%s", action_upper, YAF_G(name_separator), "Action");
-		} else {
-			class_len = spprintf(&class, 0, "%s%s%s", "Action", YAF_G(name_separator), action_upper);
-		}
-
-		class_lowercase = zend_str_tolower_dup(class, class_len);
-
-		if ((ce = zend_hash_str_find_ptr(EG(class_table), class_lowercase, class_len)) == NULL) {
-			if (!yaf_internal_autoload(action_upper, ZSTR_LEN(action), &directory)) {
-				yaf_trigger_error(YAF_ERR_NOTFOUND_ACTION, "Failed opening action script %s: %s", directory, strerror(errno));
-
-				efree(class);
-				efree(action_upper);
-				efree(class_lowercase);
-				efree(directory);
-				return NULL;
-			} else if ((ce = zend_hash_str_find_ptr(EG(class_table), class_lowercase, class_len)) == NULL)  {
-				yaf_trigger_error(YAF_ERR_AUTOLOAD_FAILED, "Could find class %s in action script %s", class, directory);
-
-				efree(class);
-				efree(action_upper);
-				efree(class_lowercase);
-				efree(directory);
-				return NULL;
-			} else if (!instanceof_function(ce, yaf_action_ce)) {
-				yaf_trigger_error(YAF_ERR_TYPE_ERROR, "Action must be an instance of %s", ZSTR_VAL(yaf_action_ce->name));
-
-				efree(class);
-				efree(action_upper);
-				efree(class_lowercase);
-				efree(directory);
-				return NULL;
-			}
-		}
-
-		efree(class);
-		efree(action_upper);
-		efree(class_lowercase);
-		efree(directory);
-
-		return ce;
-	} else
-/* }}} */
-	{
+	} else {
 		yaf_trigger_error(YAF_ERR_NOTFOUND_ACTION,
 				"There is no method %s%s in %s", ZSTR_VAL(action), "Action", ZSTR_VAL(Z_OBJCE_P(controller)->name));
 	}
@@ -588,7 +526,7 @@ int yaf_dispatcher_handle(yaf_dispatcher_t *dispatcher, yaf_request_t *request,
 
 			/* @TODO: Magic __call supports? */
 			if ((fptr = zend_hash_find_ptr(&((ce)->function_table), func_name)) != NULL) {
-				uint count = 0;
+				unsigned count = 0;
 				zval *call_args = NULL;
 				executor = &icontroller;
 				if (fptr->common.num_args) {
@@ -625,7 +563,7 @@ int yaf_dispatcher_handle(yaf_dispatcher_t *dispatcher, yaf_request_t *request,
 								YAF_ACTION_EXECUTOR_NAME, sizeof(YAF_ACTION_EXECUTOR_NAME) - 1))) {
 				zval *call_args;
 				yaf_action_t iaction;
-				uint count = 0;
+				unsigned count = 0;
 
 				zend_string_release(func_name);
 
@@ -667,6 +605,7 @@ int yaf_dispatcher_handle(yaf_dispatcher_t *dispatcher, yaf_request_t *request,
 			} else {
 				zend_string_release(func_name);
 				zval_ptr_dtor(&icontroller);
+				zval_ptr_dtor(&action);
 				return 0;
 			}
 
@@ -792,7 +731,7 @@ void yaf_dispatcher_exception_handler(yaf_dispatcher_t *dispatcher, yaf_request_
 					dispatcher, ZEND_STRL(YAF_DISPATCHER_PROPERTY_NAME_MODULE), 1, NULL);
 			/* failover to default module error catcher */
 			zend_update_property(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_MODULE), m);
-			EG(exception) = NULL;
+			zend_clear_exception();
 			(void)yaf_dispatcher_handle(dispatcher, request, response, view);
 		}
 	}
@@ -835,7 +774,7 @@ yaf_response_t *yaf_dispatcher_dispatch(yaf_dispatcher_t *dispatcher, zval *resp
 	zval *return_response, *plugins, *view, rv;
 	yaf_response_t *response;
 	yaf_request_t *request;
-	uint nesting = YAF_G(forward_limit);
+	unsigned nesting = YAF_G(forward_limit);
 
 	response = yaf_response_instance(response_ptr, sapi_module.name);
 	request	= zend_read_property(yaf_dispatcher_ce,
@@ -914,6 +853,28 @@ yaf_response_t *yaf_dispatcher_dispatch(yaf_dispatcher_t *dispatcher, zval *resp
 }
 /* }}} */
 
+static void yaf_dispatcher_set_default_method(INTERNAL_FUNCTION_PARAMETERS, const char *name, size_t name_length, zend_bool opt) /* {{{ */ {
+	zend_string *value;
+	zend_string *lc_name;
+	zval *self = getThis();
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "S", &value) == FAILURE) {
+		return;
+	}
+
+	if ((opt == YAF_DISPATCHER_CHECK_MODULE) && !yaf_application_is_module_name(value)) {
+		RETURN_FALSE;
+	}
+
+	lc_name = zend_string_init(ZSTR_VAL(value), ZSTR_LEN(value), 0);
+	*(ZSTR_VAL(lc_name)) = toupper(*(ZSTR_VAL(lc_name)));
+	zend_update_property_str(yaf_dispatcher_ce, self, name, name_length, lc_name);
+	zend_string_release(lc_name);
+
+	RETURN_ZVAL(self, 1, 0);
+}
+/* }}} */
+
 /** {{{ proto private Yaf_Dispatcher::__construct(void)
 */
 PHP_METHOD(yaf_dispatcher, __construct) {
@@ -1230,71 +1191,24 @@ PHP_METHOD(yaf_dispatcher, setView) {
 /** {{{ proto public Yaf_Dispatcher::setDefaultModule(string $name)
 */
 PHP_METHOD(yaf_dispatcher, setDefaultModule) {
-	zval *module;
-	zval *self = getThis();
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &module) == FAILURE) {
-		return;
-	}
-
-	if (IS_STRING == Z_TYPE_P(module) && Z_STRLEN_P(module)
-			&& yaf_application_is_module_name(Z_STR_P(module))) {
-		zval module_std;
-		ZVAL_STRING(&module_std, zend_str_tolower_dup(Z_STRVAL_P(module), Z_STRLEN_P(module)));
-		*Z_STRVAL(module_std) = toupper(*Z_STRVAL(module_std));
-		zend_update_property(yaf_dispatcher_ce, self, ZEND_STRL(YAF_DISPATCHER_PROPERTY_NAME_MODULE), &module_std);
-		zval_ptr_dtor(&module_std);
-
-		RETURN_ZVAL(self, 1, 0);
-	}
-
-	RETURN_FALSE;
+	(void)yaf_dispatcher_set_default_method(INTERNAL_FUNCTION_PARAM_PASSTHRU,
+			ZEND_STRL(YAF_DISPATCHER_PROPERTY_NAME_MODULE), YAF_DISPATCHER_CHECK_MODULE);
 }
 /* }}} */
 
 /** {{{ proto public Yaf_Dispatcher::setDefaultController(string $name)
 */
 PHP_METHOD(yaf_dispatcher, setDefaultController) {
-	zval *controller;
-	zval *self = getThis();
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &controller) == FAILURE) {
-		return;
-	}
-
-	if (IS_STRING == Z_TYPE_P(controller) && Z_STRLEN_P(controller)) {
-		zval controller_std;
-		ZVAL_STRING(&controller_std, zend_str_tolower_dup(Z_STRVAL_P(controller), Z_STRLEN_P(controller)));
-		*Z_STRVAL_P(&controller_std) = toupper(*Z_STRVAL(controller_std));
-		zend_update_property(yaf_dispatcher_ce, self, ZEND_STRL(YAF_DISPATCHER_PROPERTY_NAME_CONTROLLER), &controller_std);
-
-		RETURN_ZVAL(self, 1, 0);
-	}
-
-	RETURN_FALSE;
+	(void)yaf_dispatcher_set_default_method(INTERNAL_FUNCTION_PARAM_PASSTHRU,
+			ZEND_STRL(YAF_DISPATCHER_PROPERTY_NAME_CONTROLLER), YAF_DISPATCHER_CHECK_NONE);
 }
 /* }}} */
 
 /** {{{ proto public Yaf_Dispatcher::setDefaultAction(string $name)
 */
 PHP_METHOD(yaf_dispatcher, setDefaultAction) {
-	zval *action;
-	zval *self = getThis();
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &action) == FAILURE) {
-		return;
-	}
-
-	if (IS_STRING == Z_TYPE_P(action) && Z_STRLEN_P(action)) {
-		zval action_lower;
-		ZVAL_STRING(&action_lower, zend_str_tolower_dup(Z_STRVAL_P(action), Z_STRLEN_P(action)));
-		zend_update_property(yaf_dispatcher_ce, self, ZEND_STRL(YAF_DISPATCHER_PROPERTY_NAME_ACTION), &action_lower);
-		zval_ptr_dtor(&action_lower);
-
-		RETURN_ZVAL(self, 1, 0);
-	}
-
-	RETURN_FALSE;
+	(void)yaf_dispatcher_set_default_method(INTERNAL_FUNCTION_PARAM_PASSTHRU,
+			ZEND_STRL(YAF_DISPATCHER_PROPERTY_NAME_ACTION), YAF_DISPATCHER_CHECK_NONE);
 }
 /* }}} */
 
