@@ -19,6 +19,7 @@
 #endif
 
 #include "php.h"
+#include "main/SAPI.h"
 #include "standard/php_string.h" /* for php_basename */
 #include "Zend/zend_exceptions.h" /* for zend_exception_get_default */
 
@@ -26,6 +27,7 @@
 #include "yaf_request.h"
 #include "yaf_namespace.h"
 #include "yaf_exception.h"
+#include "yaf_router.h"
 
 #include "requests/yaf_request_simple.h"
 #include "requests/yaf_request_http.h"
@@ -37,7 +39,7 @@ zend_class_entry *yaf_request_ce;
 ZEND_BEGIN_ARG_INFO_EX(yaf_request_void_arginfo, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(yaf_request_set_routed_arginfo, 0, 0, 0)
+ZEND_BEGIN_ARG_INFO_EX(yaf_request_set_routed_arginfo, 0, 0, 1)
 	ZEND_ARG_INFO(0, flag)
 ZEND_END_ARG_INFO()
 
@@ -80,6 +82,10 @@ ZEND_BEGIN_ARG_INFO_EX(yaf_request_getenv_arginfo, 0, 0, 1)
 	ZEND_ARG_INFO(0, name)
 	ZEND_ARG_INFO(0, default)
 ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(yaf_request_set_dispatched_arginfo, 0, 0, 1)
+    ZEND_ARG_INFO(0, dispatched)
+ZEND_END_ARG_INFO()
 	/* }}} */
 
 yaf_request_t *yaf_request_instance(yaf_request_t *this_ptr, zend_string *request_uri) /* {{{ */ {
@@ -87,6 +93,20 @@ yaf_request_t *yaf_request_instance(yaf_request_t *this_ptr, zend_string *reques
 }
 /* }}} */
 
+const char *yaf_request_strip_base_uri(zend_string *uri, zend_string *base_uri, size_t *len) /* {{{ */ {
+	register const char *p = ZSTR_VAL(uri);
+	if (strncasecmp(p, ZSTR_VAL(base_uri), ZSTR_LEN(base_uri)) == 0) {
+		p += ZSTR_LEN(base_uri);
+		if (*p == '\0' || *p == YAF_ROUTER_URL_DELIMIETER || *(--p) == YAF_ROUTER_URL_DELIMIETER) {
+			*len = ZSTR_LEN(uri) - (p - ZSTR_VAL(uri));
+			return p;
+		}
+	}
+	*len = ZSTR_LEN(uri);
+	return ZSTR_VAL(uri);
+}
+/* }}} */
+
 int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend_string *request_uri) /* {{{ */ {
 	if (base_uri == NULL) {
 		zend_string *basename = NULL;
@@ -103,7 +123,7 @@ int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend
 				if (script_name && EXPECTED(IS_STRING == Z_TYPE_P(script_name))) {
 					zend_string	*script = php_basename(Z_STRVAL_P(script_name), Z_STRLEN_P(script_name), NULL, 0);
 
-					if (strncmp(ZSTR_VAL(file_name), ZSTR_VAL(script), ZSTR_LEN(file_name)) == 0) {
+					if (memcpy(ZSTR_VAL(file_name), ZSTR_VAL(script), ZSTR_LEN(file_name)) == 0) {
 						basename = zend_string_copy(Z_STR_P(script_name));
 						zend_string_release(file_name);
 						zend_string_release(script);
@@ -115,7 +135,7 @@ int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend
 				phpself_name = yaf_request_query_str(YAF_GLOBAL_VARS_SERVER, "PHP_SELF", sizeof("PHP_SELF") - 1);
 				if (phpself_name && EXPECTED(IS_STRING == Z_TYPE_P(phpself_name))) {
 					zend_string *phpself = php_basename(Z_STRVAL_P(phpself_name), Z_STRLEN_P(phpself_name), NULL, 0);
-					if (strncmp(ZSTR_VAL(file_name), ZSTR_VAL(phpself), ZSTR_LEN(file_name)) == 0) {
+					if (memcpy(ZSTR_VAL(file_name), ZSTR_VAL(phpself), ZSTR_LEN(file_name)) == 0) {
 						basename = zend_string_copy(Z_STR_P(phpself_name));
 						zend_string_release(file_name);
 						zend_string_release(phpself);
@@ -127,7 +147,7 @@ int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend
 				orig_name = yaf_request_query_str(YAF_GLOBAL_VARS_SERVER, "ORIG_SCRIPT_NAME", sizeof("ORIG_SCRIPT_NAME") - 1);
 				if (orig_name && IS_STRING == Z_TYPE_P(orig_name)) {
 					zend_string *orig = php_basename(Z_STRVAL_P(orig_name), Z_STRLEN_P(orig_name), NULL, 0);
-					if (strncmp(ZSTR_VAL(file_name), ZSTR_VAL(orig), ZSTR_LEN(file_name)) == 0) {
+					if (memcpy(ZSTR_VAL(file_name), ZSTR_VAL(orig), ZSTR_LEN(file_name)) == 0) {
 						basename = zend_string_copy(Z_STR_P(orig_name));
 						zend_string_release(file_name);
 						zend_string_release(orig);
@@ -139,145 +159,102 @@ int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend
 			}
 		} while (0);
 
-		if (basename && strncmp(ZSTR_VAL(request_uri), ZSTR_VAL(basename), ZSTR_LEN(basename)) == 0) {
-			if (*(ZSTR_VAL(basename) + ZSTR_LEN(basename) - 1) == '/') {
-				zend_string *garbage = basename;
-				basename = zend_string_init(ZSTR_VAL(basename), ZSTR_LEN(basename) - 1, 0);
-				zend_string_release(garbage);
-			}
-			zend_update_property_str(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_BASE), basename);
-			zend_string_release(basename);
-			return 1;
-		} else if (basename) {
-			zend_string *dir = zend_string_init(ZSTR_VAL(basename), ZSTR_LEN(basename), 0); /* php_dirname might alter the string */
-
-			ZSTR_LEN(dir) = php_dirname(ZSTR_VAL(dir), ZSTR_LEN(dir));
-			if (*(ZSTR_VAL(dir) + ZSTR_LEN(dir) - 1) == '/') {
-				--ZSTR_LEN(dir);
-			}
-
-			if (ZSTR_LEN(dir)) {
-				if (strncmp(ZSTR_VAL(request_uri), ZSTR_VAL(dir), ZSTR_LEN(dir)) == 0) {
-					zend_update_property_str(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_BASE), dir);
-					zend_string_release(dir);
+		if (basename) {
+			if (memcpy(ZSTR_VAL(request_uri), ZSTR_VAL(basename), ZSTR_LEN(basename)) == 0) {
+				if (ZSTR_VAL(basename)[ZSTR_LEN(basename) - 1] == '/') {
+					zend_string *sanitized_uri = zend_string_init(ZSTR_VAL(basename), ZSTR_LEN(basename) - 1, 0);
 					zend_string_release(basename);
-					return 1;
+					basename = sanitized_uri;
 				}
-			}
-			zend_string_release(dir);
-			zend_string_release(basename);
-		}
+				zend_update_property_str(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_BASE), basename);
+				return 1;
+			} else {
+				zend_string *dir = zend_string_init(ZSTR_VAL(basename), ZSTR_LEN(basename), 0); /* php_dirname might alter the string */
 
-		zend_update_property_string(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_BASE), "");
+				zend_string_release(basename);
+				ZSTR_LEN(dir) = php_dirname(ZSTR_VAL(dir), ZSTR_LEN(dir));
+				if (*(ZSTR_VAL(dir) + ZSTR_LEN(dir) - 1) == '/') {
+					ZSTR_VAL(dir)[ZSTR_LEN(dir) - 1] = '\0';
+					ZSTR_LEN(dir)--;
+				}
+				if (ZSTR_LEN(dir)) {
+					if (memcpy(ZSTR_VAL(request_uri), ZSTR_VAL(dir), ZSTR_LEN(dir)) == 0) {
+						zend_update_property_str(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_BASE), dir);
+						zend_string_release(dir);
+						return 1;
+					}
+				}
+				zend_string_release(dir);
+			}
+		}
+		zend_update_property_str(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_BASE), ZSTR_EMPTY_ALLOC());
 	} else {
+		zend_string *sanitized_uri = NULL;
+		if (UNEXPECTED(ZSTR_VAL(base_uri)[ZSTR_LEN(base_uri) - 1] == '/')) {
+			sanitized_uri = zend_string_init(ZSTR_VAL(base_uri), ZSTR_LEN(base_uri) - 1, 0);
+			base_uri = sanitized_uri;
+		}
 		zend_update_property_str(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_BASE), base_uri);
+		if (UNEXPECTED(sanitized_uri)) {
+			zend_string_release(sanitized_uri);
+		}
 	}
 	return 1;
 }
 /* }}} */
 
-zval *yaf_request_query_ex(uint type, zend_bool fetch_type, void *name, size_t len) /* {{{ */ {
-	zval *carrier = NULL, *ret;
-
+static inline zval* yaf_request_fetch_container(unsigned type) /* {{{ */ {
 	zend_bool jit_initialization = PG(auto_globals_jit);
 
-	/* for phpunit test requirements */
-#if PHP_YAF_DEBUG
 	switch (type) {
 		case YAF_GLOBAL_VARS_POST:
-			carrier = zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_POST"));
-			break;
+			return zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_POST"));
 		case YAF_GLOBAL_VARS_GET:
-			carrier = zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_GET"));
-			break;
+			return zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_GET"));
 		case YAF_GLOBAL_VARS_COOKIE:
-			carrier = zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_COOKIE"));
-			break;
+			return zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_COOKIE"));
+		case YAF_GLOBAL_VARS_FILES:
+			return zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_FILES"));
 		case YAF_GLOBAL_VARS_SERVER:
 			if (jit_initialization) {
-				zend_string *server_str = zend_string_init("_SERVER", sizeof("_SERVER") - 1, 0);
-				zend_is_auto_global(server_str);
-				zend_string_release(server_str);
+				zend_is_auto_global_str(ZEND_STRL("_SERVER"));
 			}
-			carrier = zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_SERVER"));
-			break;
-		case YAF_GLOBAL_VARS_ENV:
-			if (jit_initialization) {
-				zend_string *env_str = zend_string_init("_ENV", sizeof("_ENV") - 1, 0);
-				zend_is_auto_global(env_str);
-				zend_string_release(env_str);
-			}
-			carrier = &PG(http_globals)[YAF_GLOBAL_VARS_ENV];
-			break;
-		case YAF_GLOBAL_VARS_FILES:
-			carrier = &PG(http_globals)[YAF_GLOBAL_VARS_FILES];
-			break;
+			return zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_SERVER"));
 		case YAF_GLOBAL_VARS_REQUEST:
 			if (jit_initialization) {
-				zend_string *request_str = zend_string_init("_REQUEST", sizeof("_REQUEST") - 1, 0);
-				zend_is_auto_global(request_str);
-				zend_string_release(request_str);
+				zend_is_auto_global_str(ZEND_STRL("_REQUEST"));
 			}
-			carrier = zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_REQUEST"));
-			break;
-		default:
-			break;
-	}
-#else
-	switch (type) {
-		case YAF_GLOBAL_VARS_POST:
-		case YAF_GLOBAL_VARS_GET:
-		case YAF_GLOBAL_VARS_FILES:
-		case YAF_GLOBAL_VARS_COOKIE:
-			carrier = &PG(http_globals)[type];
-			break;
+			return zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_REQUEST"));
 		case YAF_GLOBAL_VARS_ENV:
 			if (jit_initialization) {
-				zend_string *env_str = zend_string_init("_ENV", sizeof("_ENV") - 1, 0);
-				zend_is_auto_global(env_str);
-				zend_string_release(env_str);
+				zend_is_auto_global_str(ZEND_STRL("_ENV"));
 			}
-			carrier = &PG(http_globals)[type];
-			break;
-		case YAF_GLOBAL_VARS_SERVER:
-			if (jit_initialization) {
-				zend_string *server_str = zend_string_init("_SERVER", sizeof("_SERVER") - 1, 0);
-				zend_is_auto_global(server_str);
-				zend_string_release(server_str);
-			}
-			carrier = &PG(http_globals)[type];
-			break;
-		case YAF_GLOBAL_VARS_REQUEST:
-			if (jit_initialization) {
-				zend_string *request_str = zend_string_init("_REQUEST", sizeof("_REQUEST") - 1, 0);
-				zend_is_auto_global(request_str);
-				zend_string_release(request_str);
-			}
-			carrier = zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_REQUEST"));
-			break;
+			return zend_hash_str_find(&EG(symbol_table), ZEND_STRL("_ENV"));
 		default:
-			break;
+			return NULL;
 	}
-#endif
+}
+/* }}} */
 
-	if (!carrier) {
+zval *yaf_request_query_str(unsigned type, const char *name, size_t len) /* {{{ */ {
+	zval *container = yaf_request_fetch_container(type);
+
+	if (UNEXPECTED(!container)) {
 		return NULL;
 	}
 
-	if (!name) {
-		return carrier;
+	return zend_hash_str_find(Z_ARRVAL_P(container), name, len);
+}
+/* }}} */
+
+zval *yaf_request_query(unsigned type, zend_string *name) /* {{{ */ {
+	zval *container = yaf_request_fetch_container(type);
+
+	if (UNEXPECTED(!container)) {
+		return NULL;
 	}
 
-	if (EXPECTED(fetch_type)) {
-		if ((ret = zend_hash_find(Z_ARRVAL_P(carrier), (zend_string *)name)) == NULL) {
-			return NULL;
-		}
-	} else {
-		if ((ret = zend_hash_str_find(Z_ARRVAL_P(carrier), (char *)name, len)) == NULL) {
-			return NULL;
-		}
-	}
-	return ret;
+	return zend_hash_find(Z_ARRVAL_P(container), name);
 }
 /* }}} */
 
@@ -298,16 +275,16 @@ zval *yaf_request_get_language(yaf_request_t *instance, zval *accept_language) /
 		} else if (UNEXPECTED(IS_STRING != Z_TYPE_P(accept_langs) || !Z_STRLEN_P(accept_langs))) {
 			return NULL;
 		} else {
-			char  	*ptrptr, *seg;
-			uint	prefer_len = 0;
+			char *ptrptr, *seg;
+			unsigned prefer_len = 0;
 			double	max_qvlaue = 0;
-			char 	*prefer = NULL;
-			char  	*langs = estrndup(Z_STRVAL_P(accept_langs), Z_STRLEN_P(accept_langs));
+			char *prefer = NULL;
+			char *langs = estrndup(Z_STRVAL_P(accept_langs), Z_STRLEN_P(accept_langs));
 
 			seg = php_strtok_r(langs, ",", &ptrptr);
-			while(seg) {
+			while (seg) {
 				char *qvalue;
-				while (*(seg) == ' ') { 
+				while (*(seg) == ' ') {
 					seg++;
 				}
 				/* Accept-Language: da, en-gb;q=0.8, en;q=0.7 */
@@ -356,8 +333,7 @@ int yaf_request_is_routed(yaf_request_t *request) /* {{{ */{
 /* }}} */
 
 int yaf_request_is_dispatched(yaf_request_t *request) /* {{{ */ {
-	zval *dispatched = zend_read_property(yaf_request_ce,
-			request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_STATE), 1, NULL);
+	zval *dispatched = zend_read_property(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_STATE), 1, NULL);
 	return Z_TYPE_P(dispatched) == IS_TRUE ? 1 : 0;
 }
 /* }}} */
@@ -368,7 +344,10 @@ void yaf_request_set_dispatched(yaf_request_t *instance, int flag) /* {{{ */ {
 /* }}} */
 
 void yaf_request_set_routed(yaf_request_t *request, int flag) /* {{{ */ {
-	zend_update_property_bool(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_ROUTED), flag);
+	zval *route = zend_read_property(yaf_request_ce, request,  ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_ROUTED), 0, NULL);
+	if ((flag && Z_TYPE_P(route) == IS_FALSE) || (!flag && Z_TYPE_P(route) == IS_TRUE)) {
+		zend_update_property_bool(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_ROUTED), flag);
+	}
 }
 /* }}} */
 
@@ -398,6 +377,17 @@ zval * yaf_request_get_param(yaf_request_t *request, zend_string *key) /* {{{ */
 }
 /* }}} */
 
+const char *yaf_request_get_request_method(void) /* {{{ */ {
+	if (SG(request_info).request_method) {
+		return SG(request_info).request_method;
+	} else if (strncasecmp(sapi_module.name, "cli", 3) == 0) {
+		return "CLI";
+	} else {
+		return "UNKNOW";
+	}
+}
+/* }}} */
+
 /** {{{ proto public Yaf_Request_Abstract::isGet(void)
 */
 YAF_REQUEST_IS_METHOD(Get);
@@ -547,7 +537,7 @@ PHP_METHOD(yaf_request, setActionName) {
 /** {{{ proto public Yaf_Request_Abstract::setParam(mixed $value)
 */
 PHP_METHOD(yaf_request, setParam) {
-	uint argc;
+	unsigned argc;
 	yaf_request_t *self	= getThis();
 
 	argc = ZEND_NUM_ARGS();
@@ -651,11 +641,18 @@ PHP_METHOD(yaf_request, isDispatched) {
 }
 /* }}} */
 
-/** {{{ proto public Yaf_Request_Abstract::setDispatched(void)
+/** {{{ proto public Yaf_Request_Abstract::setDispatched(bool $dispatched = true)
 */
 PHP_METHOD(yaf_request, setDispatched) {
-	yaf_request_set_dispatched(getThis(), 1);
-	RETURN_TRUE;
+	zend_bool state = 1;
+ 	
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|b", &state) == FAILURE){
+		return;
+	}
+	
+	yaf_request_set_dispatched(getThis(), state);
+
+	RETURN_ZVAL(getThis(), 1, 0);
 }
 /* }}} */
 
@@ -718,10 +715,16 @@ PHP_METHOD(yaf_request, isRouted) {
 }
 /* }}} */
 
-/** {{{ proto public Yaf_Request_Abstract::setRouted(void)
+/** {{{ proto public Yaf_Request_Abstract::setRouted(bool $routed = true)
 */
 PHP_METHOD(yaf_request, setRouted) {
-	yaf_request_set_routed(getThis(), 1);
+	zend_bool state = 1;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|b", &state) == FAILURE){
+		return;
+	}
+
+	yaf_request_set_routed(getThis(), state);
 	RETURN_ZVAL(getThis(), 1, 0);
 }
 /* }}} */
@@ -757,7 +760,7 @@ zend_function_entry yaf_request_methods[] = {
 	PHP_ME(yaf_request, getRequestUri, yaf_request_void_arginfo, ZEND_ACC_PUBLIC)
 	PHP_ME(yaf_request, setRequestUri, yaf_request_set_request_uri_arginfo, ZEND_ACC_PUBLIC)
 	PHP_ME(yaf_request, isDispatched, yaf_request_void_arginfo, ZEND_ACC_PUBLIC)
-	PHP_ME(yaf_request, setDispatched, yaf_request_void_arginfo, ZEND_ACC_PUBLIC)
+	PHP_ME(yaf_request, setDispatched, yaf_request_set_dispatched_arginfo, ZEND_ACC_PUBLIC)
 	PHP_ME(yaf_request, isRouted, yaf_request_void_arginfo, ZEND_ACC_PUBLIC)
 	PHP_ME(yaf_request, setRouted, yaf_request_set_routed_arginfo, ZEND_ACC_PUBLIC)
 	{NULL, NULL, NULL}
@@ -771,7 +774,7 @@ YAF_STARTUP_FUNCTION(request){
 
 	YAF_INIT_CLASS_ENTRY(ce, "Yaf_Request_Abstract", "Yaf\\Request_Abstract", yaf_request_methods);
 	yaf_request_ce 			= zend_register_internal_class_ex(&ce, NULL);
-	yaf_request_ce->ce_flags = ZEND_ACC_EXPLICIT_ABSTRACT_CLASS;
+	yaf_request_ce->ce_flags |= ZEND_ACC_EXPLICIT_ABSTRACT_CLASS;
 
 	zend_declare_property_null(yaf_request_ce, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_MODULE), ZEND_ACC_PUBLIC);
 	zend_declare_property_null(yaf_request_ce, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_CONTROLLER), ZEND_ACC_PUBLIC);
